@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-'''
+"""
 channelarchiver is a library for retrieving data from an EPICS Channel
 Archiver. It does the hard work of figuring out which archives contain
 the appropriate data for times and channels you specify.
@@ -15,13 +15,14 @@ Example usage:
     >>> lifetime, current = archiver.get(channels, start, end)
     >>> lifetime.values
     [25.8, 25.7, ..., 26.1]
-'''
 
-
-__title__ = 'channelarchiver'
-__version_info__ = (0, 0, 3)
-__version__ = '.'.join(map(str, __version_info__))
-__license__ = 'MIT'
+"""
 
 from .channelarchiver import Archiver
 from . import codes
+
+
+__title__ = "channelarchiver"
+__version__ = "1.0.0"
+__license__ = "MIT"
+__all__ = [Archiver, codes]
